@@ -540,8 +540,6 @@ class SOStringLikeCol(SOCol):
     def _sybaseType(self):
         self._check_case_sensitive("SYBASE")
         type = self._sqlType()
-        if not self.notNone and not self.alternateID:
-            type += ' NULL'
         return type
 
     def _mssqlType(self):
@@ -556,8 +554,6 @@ class SOStringLikeCol(SOCol):
             type = 'VARCHAR(%i)' % self.length
         else:
             type = 'CHAR(%i)' % self.length
-        if not self.notNone and not self.alternateID:
-            type += ' NULL'
         return type
 
     def _firebirdType(self):
@@ -594,13 +590,19 @@ class StringValidator(SOValidator):
         if self.dataType and isinstance(value, self.dataType):
             return value
         if isinstance(value,
-                      (str, buffer_type, binaryType,
+                      (str, bytes, buffer_type, binaryType,
                        sqlbuilder.SQLExpression)):
             return value
         if hasattr(value, '__unicode__'):
             return unicode(value).encode(dbEncoding)
-        if dbName == 'mysql' and not PY2 and isinstance(value, bytes):
-            return value.decode('ascii', errors='surrogateescape')
+        if dbName == 'mysql':
+            if isinstance(value, bytearray):
+                if PY2:
+                    return bytes(value)
+                else:
+                    return value.decode(dbEncoding, errors='surrogateescape')
+            if not PY2 and isinstance(value, bytes):
+                return value.decode('ascii', errors='surrogateescape')
         raise validators.Invalid(
             "expected a str in the StringCol '%s', got %s %r instead" % (
                 self.name, type(value), value), value, state)
@@ -621,7 +623,6 @@ class StringCol(Col):
 
 class NQuoted(sqlbuilder.SQLExpression):
     def __init__(self, value):
-        assert isinstance(value, unicode_type)
         self.value = value
 
     def __hash__(self):
@@ -662,6 +663,8 @@ class UnicodeStringValidator(SOValidator):
                 pass
             else:
                 if connection.dbName == 'mssql':
+                    if PY2:
+                        value = value.encode(self.getDbEncoding(state))
                     return NQuoted(value)
             return value.encode(self.getDbEncoding(state))
         if hasattr(value, '__unicode__'):
@@ -786,8 +789,18 @@ class BoolValidator(SOValidator):
             return None
         if isinstance(value, (bool, sqlbuilder.SQLExpression)):
             return value
-        if isinstance(value, (int, long)) or hasattr(value, '__nonzero__'):
+        if PY2 and hasattr(value, '__nonzero__') \
+                or not PY2 and hasattr(value, '__bool__'):
             return bool(value)
+        try:
+            connection = state.connection or state.soObject._connection
+        except AttributeError:
+            pass
+        else:
+            if connection.dbName == 'postgres' and \
+                    connection.driver in ('odbc', 'pyodbc', 'pypyodbc') and \
+                    isinstance(value, string_type):
+                return bool(int(value))
         raise validators.Invalid(
             "expected a bool or an int in the BoolCol '%s', "
             "got %s %r instead" % (
@@ -889,11 +902,15 @@ class SOKeyCol(SOCol):
         return self.key_type[self._idType()]
 
     def _sybaseType(self):
-        key_type = {int: "NUMERIC(18,0) NULL", str: "TEXT"}
+        key_type = {int: "NUMERIC(18,0)", str: "TEXT"}
         return key_type[self._idType()]
 
     def _mssqlType(self):
-        key_type = {int: "INT NULL", str: "TEXT"}
+        key_type = {int: "INT", str: "TEXT"}
+        return key_type[self._idType()]
+
+    def _firebirdType(self):
+        key_type = {int: "INT", str: "VARCHAR(255)"}
         return key_type[self._idType()]
 
 
@@ -1751,12 +1768,14 @@ class BinaryValidator(SOValidator):
             dbName = connection.dbName
             binaryType = connection._binaryType
         if isinstance(value, str):
+            if not PY2 and dbName == "mysql":
+                value = value.encode('ascii', errors='surrogateescape')
             if dbName == "sqlite":
                 if not PY2:
                     value = bytes(value, 'ascii')
                 value = connection.module.decode(value)
-            if dbName == "mysql" and not PY2:
-                value = value.encode('ascii', errors='surrogateescape')
+            return value
+        if isinstance(value, bytes):
             return value
         if isinstance(value, (buffer_type, binaryType)):
             cachedValue = self._cachedValue
@@ -1835,7 +1854,10 @@ class PickleValidator(BinaryValidator):
             return None
         if isinstance(value, unicode_type):
             dbEncoding = self.getDbEncoding(state, default='ascii')
-            value = value.encode(dbEncoding)
+            if PY2:
+                value = value.encode(dbEncoding)
+            else:
+                value = value.encode(dbEncoding, errors='surrogateescape')
         if isinstance(value, bytes):
             return pickle.loads(value)
         raise validators.Invalid(
@@ -1879,8 +1901,12 @@ class UuidValidator(SOValidator):
     def to_python(self, value, state):
         if value is None:
             return None
+        if PY2 and isinstance(value, unicode):
+            value = value.encode('ascii')
         if isinstance(value, str):
             return UUID(value)
+        if isinstance(value, UUID):
+            return value
         raise validators.Invalid(
             "expected string in the UuidCol '%s', "
             "got %s %r instead" % (
@@ -1916,6 +1942,8 @@ class UuidCol(Col):
 class JsonbValidator(SOValidator):
 
     def to_python(self, value, state):
+        if isinstance(value, string_type):
+            return json.loads(value)
         return value
 
     def from_python(self, value, state):
