@@ -210,12 +210,6 @@ class DBConnection:
     @staticmethod
     def _parseURI(uri):
         parsed = urlparse(uri)
-        if sys.version_info[0:2] == (2, 6):
-            # In python 2.6, urlparse only parses the uri completely
-            # for certain schemes, so we force the scheme to
-            # something that will be parsed correctly
-            scheme = parsed.scheme
-            parsed = urlparse(uri.replace(scheme, 'http', 1))
         host, path = parsed.hostname, parsed.path
         user, password, port = None, None, None
         if parsed.username:
@@ -734,6 +728,15 @@ class DBAPI(DBConnection):
         """
         raise NotImplementedError
 
+    def make_odbc_conn_str(self, odb_source, db, host=None, port=None,
+                           user=None, password=None):
+        odbc_conn_parts = ['Driver={%s}' % odb_source]
+        for odbc_keyword, value in \
+                zip(self.odbc_keywords, (host, port, user, password, db)):
+            if value is not None:
+                odbc_conn_parts.append('%s=%s' % (odbc_keyword, value))
+        self.odbc_conn_str = ';'.join(odbc_conn_parts)
+
 
 class Iteration(object):
 
