@@ -1,11 +1,15 @@
+from functools import reduce
+
 from sqlobject import dbconnection
 from sqlobject import classregistry
 from sqlobject import events
 from sqlobject import sqlbuilder
 from sqlobject.col import StringCol, ForeignKey
 from sqlobject.main import sqlmeta, SQLObject, SelectResults, \
-   makeProperties, unmakeProperties, getterName, setterName
-import iteration
+    makeProperties, unmakeProperties, getterName, setterName
+from sqlobject.compat import string_type
+from . import iteration
+
 
 def tablesUsedSet(obj, db):
     if hasattr(obj, "tablesUsedSet"):
@@ -23,11 +27,12 @@ class InheritableSelectResults(SelectResults):
     IterationClass = iteration.InheritableIteration
 
     def __init__(self, sourceClass, clause, clauseTables=None,
-            inheritedTables=None, **ops):
+                 inheritedTables=None, **ops):
         if clause is None or isinstance(clause, str) and clause == 'all':
             clause = sqlbuilder.SQLTrueClause
 
-        dbName = (ops.get('connection',None) or sourceClass._connection).dbName
+        dbName = (ops.get('connection', None) or
+                  sourceClass._connection).dbName
 
         tablesSet = tablesUsedSet(clause, dbName)
         tablesSet.add(str(sourceClass.sqlmeta.table))
@@ -35,21 +40,21 @@ class InheritableSelectResults(SelectResults):
         if inheritedTables:
             for tableName in inheritedTables:
                 tablesSet.add(str(tableName))
-        if orderBy and not isinstance(orderBy, basestring):
+        if orderBy and not isinstance(orderBy, string_type):
             tablesSet.update(tablesUsedSet(orderBy, dbName))
-        #DSM: if this class has a parent, we need to link it
-        #DSM: and be sure the parent is in the table list.
-        #DSM: The following code is before clauseTables
-        #DSM: because if the user uses clauseTables
-        #DSM: (and normal string SELECT), he must know what he wants
-        #DSM: and will do himself the relationship between classes.
+        # DSM: if this class has a parent, we need to link it
+        # DSM: and be sure the parent is in the table list.
+        # DSM: The following code is before clauseTables
+        # DSM: because if the user uses clauseTables
+        # DSM: (and normal string SELECT), he must know what he wants
+        # DSM: and will do himself the relationship between classes.
         if not isinstance(clause, str):
             tableRegistry = {}
             allClasses = classregistry.registry(
                 sourceClass.sqlmeta.registry).allClasses()
             for registryClass in allClasses:
                 if str(registryClass.sqlmeta.table) in tablesSet:
-                    #DSM: By default, no parents are needed for the clauses
+                    # DSM: By default, no parents are needed for the clauses
                     tableRegistry[registryClass] = registryClass
             tableRegistryCopy = tableRegistry.copy()
             for childClass in tableRegistryCopy:
@@ -59,60 +64,69 @@ class InheritableSelectResults(SelectResults):
                 while currentClass:
                     if currentClass in tableRegistryCopy:
                         if currentClass in tableRegistry:
-                            #DSM: Remove this class as it is a parent one
-                            #DSM: of a needed children
+                            # DSM: Remove this class as it is a parent one
+                            # DSM: of a needed children
                             del tableRegistry[currentClass]
-                        #DSM: Must keep the last parent needed
-                        #DSM: (to limit the number of join needed)
+                        # DSM: Must keep the last parent needed
+                        # DSM: (to limit the number of join needed)
                         tableRegistry[childClass] = currentClass
                     currentClass = currentClass.sqlmeta.parentClass
-            #DSM: Table registry contains only the last children
-            #DSM: or standalone classes
+            # DSM: Table registry contains only the last children
+            # DSM: or standalone classes
             parentClause = []
             for (currentClass, minParentClass) in tableRegistry.items():
                 while (currentClass != minParentClass) \
-                and currentClass.sqlmeta.parentClass:
+                        and currentClass.sqlmeta.parentClass:
                     parentClass = currentClass.sqlmeta.parentClass
                     parentClause.append(currentClass.q.id == parentClass.q.id)
                     currentClass = parentClass
                     tablesSet.add(str(currentClass.sqlmeta.table))
             clause = reduce(sqlbuilder.AND, parentClause, clause)
 
-        super(InheritableSelectResults, self).__init__(sourceClass,
-            clause, clauseTables, **ops)
+        super(InheritableSelectResults, self).__init__(
+            sourceClass, clause, clauseTables, **ops)
 
     def accumulateMany(self, *attributes, **kw):
         if kw.get("skipInherited"):
-            return super(InheritableSelectResults, self).accumulateMany(*attributes)
+            return super(InheritableSelectResults, self).\
+                accumulateMany(*attributes)
         tables = []
         for func_name, attribute in attributes:
-           if not isinstance(attribute, basestring):
+            if not isinstance(attribute, string_type):
                 tables.append(attribute.tableName)
         clone = self.__class__(self.sourceClass, self.clause,
-                          self.clauseTables, inheritedTables=tables, **self.ops)
+                               self.clauseTables, inheritedTables=tables,
+                               **self.ops)
         return clone.accumulateMany(skipInherited=True, *attributes)
 
+
 class InheritableSQLMeta(sqlmeta):
     @classmethod
-    def addColumn(sqlmeta, columnDef, changeSchema=False, connection=None, childUpdate=False):
+    def addColumn(sqlmeta, columnDef, changeSchema=False, connection=None,
+                  childUpdate=False):
         soClass = sqlmeta.soClass
-        #DSM: Try to add parent properties to the current class
-        #DSM: Only do this once if possible at object creation and once for
-        #DSM: each new dynamic column to refresh the current class
+        # DSM: Try to add parent properties to the current class
+        # DSM: Only do this once if possible at object creation and once for
+        # DSM: each new dynamic column to refresh the current class
         if sqlmeta.parentClass:
             for col in sqlmeta.parentClass.sqlmeta.columnList:
                 cname = col.name
-                if cname == 'childName': continue
-                if cname.endswith("ID"): cname = cname[:-2]
+                if cname == 'childName':
+                    continue
+                if cname.endswith("ID"):
+                    cname = cname[:-2]
                 setattr(soClass, getterName(cname), eval(
                     'lambda self: self._parent.%s' % cname))
                 if not col.immutable:
                     def make_setfunc(cname):
                         def setfunc(self, val):
-                            if not self.sqlmeta._creating and not getattr(self.sqlmeta, "row_update_sig_suppress", False):
-                                self.sqlmeta.send(events.RowUpdateSignal, self, {cname : val})
+                            if not self.sqlmeta._creating and \
+                               not getattr(self.sqlmeta,
+                                           "row_update_sig_suppress", False):
+                                self.sqlmeta.send(events.RowUpdateSignal, self,
+                                                  {cname: val})
 
-                            result = setattr(self._parent, cname, val)
+                            setattr(self._parent, cname, val)
                         return setfunc
 
                     setfunc = make_setfunc(cname)
@@ -122,20 +136,25 @@ class InheritableSQLMeta(sqlmeta):
                 return
 
         if columnDef:
-            super(InheritableSQLMeta, sqlmeta).addColumn(columnDef, changeSchema, connection)
+            super(InheritableSQLMeta, sqlmeta).addColumn(columnDef,
+                                                         changeSchema,
+                                                         connection)
 
-        #DSM: Update each child class if needed and existing (only for new
-        #DSM: dynamic column as no child classes exists at object creation)
+        # DSM: Update each child class if needed and existing (only for new
+        # DSM: dynamic column as no child classes exists at object creation)
         if columnDef and hasattr(soClass, "q"):
             q = getattr(soClass.q, columnDef.name, None)
         else:
             q = None
         for c in sqlmeta.childClasses.values():
-            c.sqlmeta.addColumn(columnDef, connection=connection, childUpdate=True)
-            if q: setattr(c.q, columnDef.name, q)
+            c.sqlmeta.addColumn(columnDef, connection=connection,
+                                childUpdate=True)
+            if q:
+                setattr(c.q, columnDef.name, q)
 
     @classmethod
-    def delColumn(sqlmeta, column, changeSchema=False, connection=None, childUpdate=False):
+    def delColumn(sqlmeta, column, changeSchema=False, connection=None,
+                  childUpdate=False):
         if childUpdate:
             soClass = sqlmeta.soClass
             unmakeProperties(soClass)
@@ -149,31 +168,35 @@ class InheritableSQLMeta(sqlmeta):
             delattr(soClass.q, name)
             return
 
-        super(InheritableSQLMeta, sqlmeta).delColumn(column, changeSchema, connection)
+        super(InheritableSQLMeta, sqlmeta).delColumn(column, changeSchema,
+                                                     connection)
 
-        #DSM: Update each child class if needed
-        #DSM: and delete properties for this column
+        # DSM: Update each child class if needed
+        # DSM: and delete properties for this column
         for c in sqlmeta.childClasses.values():
             c.sqlmeta.delColumn(column, changeSchema=changeSchema,
-                connection=connection, childUpdate=True)
+                                connection=connection, childUpdate=True)
 
     @classmethod
     def addJoin(sqlmeta, joinDef, childUpdate=False):
         soClass = sqlmeta.soClass
-        #DSM: Try to add parent properties to the current class
-        #DSM: Only do this once if possible at object creation and once for
-        #DSM: each new dynamic join to refresh the current class
+        # DSM: Try to add parent properties to the current class
+        # DSM: Only do this once if possible at object creation and once for
+        # DSM: each new dynamic join to refresh the current class
         if sqlmeta.parentClass:
             for join in sqlmeta.parentClass.sqlmeta.joins:
                 jname = join.joinMethodName
-                jarn  = join.addRemoveName
-                setattr(soClass, getterName(jname),
+                jarn = join.addRemoveName
+                setattr(
+                    soClass, getterName(jname),
                     eval('lambda self: self._parent.%s' % jname))
                 if hasattr(join, 'remove'):
-                    setattr(soClass, 'remove' + jarn,
+                    setattr(
+                        soClass, 'remove' + jarn,
                         eval('lambda self,o: self._parent.remove%s(o)' % jarn))
                 if hasattr(join, 'add'):
-                    setattr(soClass, 'add' + jarn,
+                    setattr(
+                        soClass, 'add' + jarn,
                         eval('lambda self,o: self._parent.add%s(o)' % jarn))
             if childUpdate:
                 makeProperties(soClass)
@@ -182,8 +205,8 @@ class InheritableSQLMeta(sqlmeta):
         if joinDef:
             super(InheritableSQLMeta, sqlmeta).addJoin(joinDef)
 
-        #DSM: Update each child class if needed and existing (only for new
-        #DSM: dynamic join as no child classes exists at object creation)
+        # DSM: Update each child class if needed and existing (only for new
+        # DSM: dynamic join as no child classes exists at object creation)
         for c in sqlmeta.childClasses.values():
             c.sqlmeta.addJoin(joinDef, childUpdate=True)
 
@@ -197,8 +220,8 @@ class InheritableSQLMeta(sqlmeta):
 
         super(InheritableSQLMeta, sqlmeta).delJoin(joinDef)
 
-        #DSM: Update each child class if needed
-        #DSM: and delete properties for this join
+        # DSM: Update each child class if needed
+        # DSM: and delete properties for this join
         for c in sqlmeta.childClasses.values():
             c.sqlmeta.delJoin(joinDef, childUpdate=True)
 
@@ -242,7 +265,7 @@ class InheritableSQLObject(SQLObject):
                 if isinstance(column, ForeignKey):
                     continue
                 setattr(cls.q, column.name,
-                    getattr(currentClass.q, column.name))
+                        getattr(currentClass.q, column.name))
             currentClass = currentClass.sqlmeta.parentClass
 
     @classmethod
@@ -263,7 +286,7 @@ class InheritableSQLObject(SQLObject):
         sqlmeta.parentClass = None
         for superclass in cls.__bases__:
             if getattr(superclass, '_inheritable', False) \
-            and (superclass.__name__ != 'InheritableSQLObject'):
+                    and (superclass.__name__ != 'InheritableSQLObject'):
                 if sqlmeta.parentClass:
                     # already have a parent class;
                     # cannot inherit from more than one
@@ -281,13 +304,16 @@ class InheritableSQLObject(SQLObject):
                 sqlmeta.childName = cls.__name__
 
     @classmethod
-    def get(cls, id, connection=None, selectResults=None, childResults=None, childUpdate=False):
+    def get(cls, id, connection=None, selectResults=None,
+            childResults=None, childUpdate=False):
 
-        val = super(InheritableSQLObject, cls).get(id, connection, selectResults)
+        val = super(InheritableSQLObject, cls).get(id, connection,
+                                                   selectResults)
 
-        #DSM: If we are updating a child, we should never return a child...
-        if childUpdate: return val
-        #DSM: If this class has a child, return the child
+        # DSM: If we are updating a child, we should never return a child...
+        if childUpdate:
+            return val
+        # DSM: If this class has a child, return the child
         if 'childName' in cls.sqlmeta.columns:
             childName = val.childName
             if childName is not None:
@@ -302,15 +328,15 @@ class InheritableSQLObject(SQLObject):
                 if not (childResults or childClass.sqlmeta.columns):
                     childResults = (None,)
                 return childClass.get(id, connection=connection,
-                    selectResults=childResults)
-        #DSM: Now, we know we are alone or the last child in a family...
-        #DSM: It's time to find our parents
+                                      selectResults=childResults)
+        # DSM: Now, we know we are alone or the last child in a family...
+        # DSM: It's time to find our parents
         inst = val
         while inst.sqlmeta.parentClass and not inst._parent:
-            inst._parent = inst.sqlmeta.parentClass.get(id,
-                connection=connection, childUpdate=True)
+            inst._parent = inst.sqlmeta.parentClass.get(
+                id, connection=connection, childUpdate=True)
             inst = inst._parent
-        #DSM: We can now return ourself
+        # DSM: We can now return ourself
         return val
 
     @classmethod
@@ -325,14 +351,15 @@ class InheritableSQLObject(SQLObject):
                     raise AttributeError(
                         "The column name 'childName' is reserved")
                 if column.name in parentCols:
-                    raise AttributeError("The column '%s' is"
-                        " already defined in an inheritable parent"
-                        % column.name)
+                    raise AttributeError(
+                        "The column '%s' is already defined "
+                        "in an inheritable parent" % column.name)
         # if this class is inheritable, add column for children distinction
         if cls._inheritable and (cls.__name__ != 'InheritableSQLObject'):
-            sqlmeta.addColumn(StringCol(name='childName',
-                # limit string length to get VARCHAR and not CLOB
-                length=255, default=None))
+            sqlmeta.addColumn(
+                StringCol(name='childName',
+                          # limit string length to get VARCHAR and not CLOB
+                          length=255, default=None))
         if not sqlmeta.columnList:
             # There are no columns - call addColumn to propagate columns
             # from parent classes to children
@@ -344,15 +371,15 @@ class InheritableSQLObject(SQLObject):
 
     def _create(self, id, **kw):
 
-        #DSM: If we were called by a children class,
-        #DSM: we must retreive the properties dictionary.
-        #DSM: Note: we can't use the ** call paremeter directly
-        #DSM: as we must be able to delete items from the dictionary
-        #DSM: (and our children must know that the items were removed!)
+        # DSM: If we were called by a children class,
+        # DSM: we must retreive the properties dictionary.
+        # DSM: Note: we can't use the ** call paremeter directly
+        # DSM: as we must be able to delete items from the dictionary
+        # DSM: (and our children must know that the items were removed!)
         if 'kw' in kw:
             kw = kw['kw']
-        #DSM: If we are the children of an inheritable class,
-        #DSM: we must first create our parent
+        # DSM: If we are the children of an inheritable class,
+        # DSM: we must first create our parent
         if self.sqlmeta.parentClass:
             parentClass = self.sqlmeta.parentClass
             new_kw = {}
@@ -370,11 +397,13 @@ class InheritableSQLObject(SQLObject):
             for col in self.sqlmeta.columnList:
                 if (col._default == sqlbuilder.NoDefault) and \
                         (col.name not in kw) and (col.foreignName not in kw):
-                    raise TypeError, "%s() did not get expected keyword argument %s" % (self.__class__.__name__, col.name)
+                    raise TypeError(
+                        "%s() did not get expected keyword argument "
+                        "%s" % (self.__class__.__name__, col.name))
 
             parent_kw['childName'] = self.sqlmeta.childName
             self._parent = parentClass(kw=parent_kw,
-                connection=self._connection)
+                                       connection=self._connection)
 
             id = self._parent.id
 
@@ -383,12 +412,13 @@ class InheritableSQLObject(SQLObject):
         try:
             super(InheritableSQLObject, self)._create(id, **kw)
         except:
-            # If we are outside a transaction and this is a child, destroy the parent
+            # If we are outside a transaction and this is a child,
+            # destroy the parent
             connection = self._connection
             if (not isinstance(connection, dbconnection.Transaction) and
                     connection.autoCommit) and self.sqlmeta.parentClass:
                 self._parent.destroySelf()
-                #TC: Do we need to do this??
+                # TC: Do we need to do this??
                 self._parent = None
             # TC: Reraise the original exception
             raise
@@ -426,7 +456,9 @@ class InheritableSQLObject(SQLObject):
                 addClause = parentClass.q.childName == cls.sqlmeta.childName
                 # if the clause was one of TRUE varians, replace it
                 if (clause is None) or (clause is sqlbuilder.SQLTrueClause) \
-                or (isinstance(clause, basestring) and (clause == 'all')):
+                        or (
+                            isinstance(clause, string_type) and
+                            (clause == 'all')):
                     clause = addClause
                 else:
                     # patch WHERE condition:
@@ -435,6 +467,7 @@ class InheritableSQLObject(SQLObject):
                     #     to build a new one if we have to replace field
                     clsID = cls.q.id
                     parentID = parentClass.q.id
+
                     def _get_patched(clause):
                         if isinstance(clause, sqlbuilder.SQLOp):
                             _patch_id_clause(clause)
@@ -442,10 +475,11 @@ class InheritableSQLObject(SQLObject):
                         elif not isinstance(clause, sqlbuilder.Field):
                             return None
                         elif (clause.tableName == clsID.tableName) \
-                        and (clause.fieldName == clsID.fieldName):
+                                and (clause.fieldName == clsID.fieldName):
                             return parentID
                         else:
                             return None
+
                     def _patch_id_clause(clause):
                         if not isinstance(clause, sqlbuilder.SQLOp):
                             return
@@ -459,7 +493,7 @@ class InheritableSQLObject(SQLObject):
                     # add childName filter
                     clause = sqlbuilder.AND(clause, addClause)
             return parentClass.select(clause, childUpdate=False,
-                *args, **kwargs)
+                                      *args, **kwargs)
         else:
             return super(InheritableSQLObject, cls).select(
                 clause, *args, **kwargs)
@@ -470,14 +504,15 @@ class InheritableSQLObject(SQLObject):
         foreignColumns = {}
         currentClass = cls
         while currentClass:
-            foreignColumns.update(dict([(column.foreignName, name)
-                for (name, column) in currentClass.sqlmeta.columns.items()
+            foreignColumns.update(dict(
+                [(column.foreignName, name)
+                    for (name, column) in currentClass.sqlmeta.columns.items()
                     if column.foreignKey
-            ]))
+                 ]))
             currentClass = currentClass.sqlmeta.parentClass
         for name, value in kw.items():
             if name in foreignColumns:
-                name = foreignColumns[name] # translate "key" to "keyID"
+                name = foreignColumns[name]  # translate "key" to "keyID"
                 if isinstance(value, SQLObject):
                     value = value.id
             currentClass = cls
@@ -485,21 +520,22 @@ class InheritableSQLObject(SQLObject):
                 try:
                     clause.append(getattr(currentClass.q, name) == value)
                     break
-                except AttributeError, err:
+                except AttributeError:
                     pass
                 currentClass = currentClass.sqlmeta.parentClass
             else:
-                raise AttributeError("'%s' instance has no attribute '%s'"
-                    % (cls.__name__, name))
+                raise AttributeError(
+                    "'%s' instance has no attribute '%s'" % (
+                        cls.__name__, name))
         if clause:
             clause = reduce(sqlbuilder.AND, clause)
         else:
-            clause = None # select all
+            clause = None  # select all
         conn = connection or cls._connection
         return cls.SelectResultsClass(cls, clause, connection=conn)
 
     def destroySelf(self):
-        #DSM: If this object has parents, recursivly kill them
+        # DSM: If this object has parents, recursivly kill them
         if hasattr(self, '_parent') and self._parent:
             self._parent.destroySelf()
         super(InheritableSQLObject, self).destroySelf()
