@@ -43,9 +43,9 @@ from .sresults import SelectResults
 from .util.threadinglocal import local
 from sqlobject.compat import PY2, with_metaclass, string_type, unicode_type
 
-if ((sys.version_info[0] == 2) and (sys.version_info[:2] < (2, 6))) or \
+if ((sys.version_info[0] == 2) and (sys.version_info[:2] < (2, 7))) or \
    ((sys.version_info[0] == 3) and (sys.version_info[:2] < (3, 4))):
-    raise ImportError("SQLObject requires Python 2.6, 2.7 or 3.4+")
+    raise ImportError("SQLObject requires Python 2.7 or 3.4+")
 
 if not PY2:
     # alias for python 3 compatability
@@ -693,9 +693,9 @@ def deprecated(message, level=1, stacklevel=2):
     if warnings_level is not None and warnings_level <= level:
         warnings.warn(message, DeprecationWarning, stacklevel=stacklevel)
 
-# if sys.version_info[:2] < (2, 6):
-#     deprecated("Support for Python 2.5 has been declared obsolete "
-#     "and will be removed in the next release of SQLObject")
+# if sys.version_info[:2] < (2, 7):
+#     deprecated("Support for Python 2.6 has been declared obsolete "
+#                "and will be removed in the next release of SQLObject")
 
 
 def setDeprecationLevel(warning=1, exception=None):
