@@ -11,13 +11,14 @@ connection creation sample::
 
     __connection__ = DBConnection.maxdbConnection(
         host=hostname, database=dbname,
-        user=user_name, password=user_password,autoCommit=1,debug=1)
+        user=user_name, password=user_password, autoCommit=1, debug=1)
 """
+
+import os
 from sqlobject.dbconnection import DBAPI
 from sqlobject import col
 
 
-
 class maxdbException(Exception):
 
     def __init__(self, value):
@@ -26,18 +27,21 @@ class maxdbException(Exception):
     def __str__(self):
         return repr(self.value)
 
+
 class LowerBoundOfSliceIsNotSupported(maxdbException):
     def __init__(self, value):
         maxdbException.__init__(self, '')
 
-class IncorrectIDStyleError(maxdbException) :
-    def __init__(self,value):
+
+class IncorrectIDStyleError(maxdbException):
+    def __init__(self, value):
         maxdbException.__init__(
             self,
             'This primary key name is not in the expected style, '
             'please rename the column to %r or switch to another style'
             % value)
 
+
 class StyleMismatchError(maxdbException):
     def __init__(self, value):
         maxdbException.__init__(
@@ -45,13 +49,16 @@ class StyleMismatchError(maxdbException):
             'The name %r is only permitted for primary key, change the '
             'column name or switch to another style' % value)
 
+
 class PrimaryKeyNotFounded(maxdbException):
     def __init__(self, value):
         maxdbException.__init__(
             self,
             "No primary key was defined on table %r" % value)
 
-SAPDBMAX_ID_LENGTH=32
+
+SAPDBMAX_ID_LENGTH = 32
+
 
 class MaxdbConnection(DBAPI):
 
@@ -59,48 +66,48 @@ class MaxdbConnection(DBAPI):
     dbName = 'maxdb'
     schemes = [dbName]
 
-    def __init__ (self, user, password, database,
-                  host='', autoCommit=1, sqlmode='internal',
-                  isolation=None, timeout=None, **kw):
+    def __init__(self, host='', port=None, user=None, password=None,
+                 database=None, autoCommit=1, sqlmode='internal',
+                 isolation=None, timeout=None, **kw):
         from sapdb import dbapi
         self.module = dbapi
+        self.host = host
+        self.port = port
+        self.user = user
+        self.password = password
+        self.db = database
         self.autoCommit = autoCommit
-        self.user      = user
-        self.password  = password
-        self.database  = database
-        self.host      = host
-        self.sqlmode   = sqlmode
+        self.sqlmode = sqlmode
         self.isolation = isolation
-        self.timeout   = timeout
+        self.timeout = timeout
 
         DBAPI.__init__(self, **kw)
 
-    def connectionFromURI(cls, uri):
-        auth, password, host, port, path, args = cls._parseURI(uri)
+    @classmethod
+    def _connectionFromParams(cls, auth, password, host, port, path, args):
         path = path.replace('/', os.path.sep)
-        return cls(host, db=path, user=auth, password=password, **args)
-    connectionFromURI = classmethod(connectionFromURI)
+        return cls(host, port, user=auth, password=password, database=path,
+                   **args)
 
-    def _getConfigParams(self,sqlmode,auto):
-        autocommit='off'
+    def _getConfigParams(self, sqlmode, auto):
+        autocommit = 'off'
         if auto:
-            autocommit='on'
+            autocommit = 'on'
         opt = {}
         opt["autocommit"] = autocommit
         opt["sqlmode"] = sqlmode
         if self.isolation:
-            opt["isolation"]=self.isolation
-        if self.timeout :
-            opt["timeout"]=self.timeout
+            opt["isolation"] = self.isolation
+        if self.timeout:
+            opt["timeout"] = self.timeout
         return opt
 
     def _setAutoCommit(self, conn, auto):
         conn.close()
-        conn.__init__(self.user, self.password, self.database,
-                      self.host,
-                      **self._getConfigParams(self.sqlmode,auto))
+        conn.__init__(self.user, self.password, self.db, self.host,
+                      **self._getConfigParams(self.sqlmode, auto))
 
-    def createSequenceName(self,table):
+    def createSequenceName(self, table):
         """
         sequence name are builded with the concatenation of the table
         name with '_SEQ' word we truncate the name of the
@@ -108,12 +115,12 @@ class MaxdbConnection(DBAPI):
         characters so that the name of the sequence does not exceed 32
         characters
         """
-        return '%s_SEQ'%(table[:SAPDBMAX_ID_LENGTH -4])
+        return '%s_SEQ' % (table[:SAPDBMAX_ID_LENGTH - 4])
 
     def makeConnection(self):
         conn = self.module.Connection(
-            self.user, self.password, self.database, self.host,
-            **self._getConfigParams(self.sqlmode,self.autoCommit))
+            self.user, self.password, self.db, self.host,
+            **self._getConfigParams(self.sqlmode, self.autoCommit))
         return conn
 
     def _queryInsertID(self, conn, soInstance, id, names, values):
@@ -121,7 +128,9 @@ class MaxdbConnection(DBAPI):
         idName = soInstance.sqlmeta.idName
         c = conn.cursor()
         if id is None:
-            c.execute('SELECT %s.NEXTVAL FROM DUAL' % (self.createSequenceName(table)))
+            c.execute(
+                'SELECT %s.NEXTVAL FROM DUAL' % (
+                    self.createSequenceName(table)))
             id = c.fetchone()[0]
         names = [idName] + names
         values = [id] + values
@@ -133,37 +142,38 @@ class MaxdbConnection(DBAPI):
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
 
-    def sqlAddLimit(cls,query,limit):
+    @classmethod
+    def sqlAddLimit(cls, query, limit):
         sql = query
-        sql = sql.replace("SELECT","SELECT ROWNO, ")
+        sql = sql.replace("SELECT", "SELECT ROWNO, ")
         if sql.find('WHERE') != -1:
             sql = sql + ' AND ' + limit
         else:
             sql = sql + 'WHERE ' + limit
         return sql
-    sqlAddLimit = classmethod(sqlAddLimit)
 
+    @classmethod
     def _queryAddLimitOffset(cls, query, start, end):
         if start:
             raise LowerBoundOfSliceIsNotSupported
         limit = ' ROWNO   <= %d ' % (end)
-        return cls.sqlAddLimit(query,limit)
-    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
-
+        return cls.sqlAddLimit(query, limit)
 
     def createTable(self, soClass):
-        #we create the table in a transaction because the addition of the
-        #table and the sequence must be atomic
+        # We create the table in a transaction because the addition of the
+        # table and the sequence must be atomic.
 
-        #i tried to use the transaction class but i get a recursion limit error
-        #t=self.transaction()
-        # t.query('CREATE TABLE %s (\n%s\n)' % \
+        # I tried to use the transaction class
+        # but I get a recursion limit error.
+        # t=self.transaction()
+        # t.query('CREATE TABLE %s (\n%s\n)' %
         #            (soClass.sqlmeta.table, self.createColumns(soClass)))
         #
-        # t.query("CREATE SEQUENCE %s" % self.createSequenceName(soClass.sqlmeta.table))
+        # t.query("CREATE SEQUENCE %s" %
+        #         self.createSequenceName(soClass.sqlmeta.table))
         # t.commit()
-        #so use transaction when the problem will be solved
-        self.query('CREATE TABLE %s (\n%s\n)' % \
+        # so use transaction when the problem will be solved
+        self.query('CREATE TABLE %s (\n%s\n)' %
                    (soClass.sqlmeta.table, self.createColumns(soClass)))
         self.query("CREATE SEQUENCE %s"
                    % self.createSequenceName(soClass.sqlmeta.table))
@@ -182,10 +192,11 @@ class MaxdbConnection(DBAPI):
     def createIndexSQL(self, soClass, index):
         return index.maxdbCreateIndexSQL(soClass)
 
-    def dropTable(self, tableName,cascade=False):
-        #we drop the table in a transaction because the removal of the
-        #table and the sequence must be atomic
-        #i tried to use the transaction class but i get a recursion limit error
+    def dropTable(self, tableName, cascade=False):
+        # We drop the table in a transaction because the removal of the
+        # table and the sequence must be atomic.
+        # I tried to use the transaction class
+        # but I get a recursion limit error.
         # try:
         #     t=self.transaction()
         #     t.query("DROP TABLE %s" % tableName)
@@ -193,7 +204,7 @@ class MaxdbConnection(DBAPI):
         #     t.commit()
         # except:
         #     t.rollback()
-        #so use transaction when the problem will be solved
+        # so use transaction when the problem will be solved
         self.query("DROP TABLE %s" % tableName)
         self.query("DROP SEQUENCE %s" % self.createSequenceName(tableName))
 
@@ -201,7 +212,9 @@ class MaxdbConnection(DBAPI):
         return 'INT NOT NULL'
 
     def tableExists(self, tableName):
-        for (table,) in self.queryAll("SELECT OBJECT_NAME FROM ALL_OBJECTS WHERE OBJECT_TYPE='TABLE'"):
+        for (table,) in self.queryAll(
+                "SELECT OBJECT_NAME FROM ALL_OBJECTS "
+                "WHERE OBJECT_TYPE='TABLE'"):
             if table.lower() == tableName.lower():
                 return True
         return False
@@ -212,7 +225,8 @@ class MaxdbConnection(DBAPI):
                     column.maxdbCreateSQL()))
 
     def delColumn(self, sqlmeta, column):
-        self.query('ALTER TABLE %s DROP COLUMN %s' % (sqlmeta.table, column.dbName))
+        self.query('ALTER TABLE %s DROP COLUMN %s' % (sqlmeta.table,
+                                                      column.dbName))
 
     GET_COLUMNS = """
     SELECT COLUMN_NAME, NULLABLE, DATA_DEFAULT, DATA_TYPE,
@@ -235,50 +249,51 @@ class MaxdbConnection(DBAPI):
 
         results = []
         keymap = {}
-        pkmap={}
-        fkData = self.queryAll(self.GET_PK_AND_FK% tableName)
-        for col, cons_type, refcol, reftable in fkData:
-            col_name= col.lower()
-            pkmap[col_name]=False
+        pkmap = {}
+        fkData = self.queryAll(self.GET_PK_AND_FK % tableName)
+        for _col, cons_type, refcol, reftable in fkData:
+            col_name = _col.lower()
+            pkmap[col_name] = False
             if cons_type == 'R':
-                keymap[col_name]=reftable.lower()
+                keymap[col_name] = reftable.lower()
 
             elif cons_type == 'P':
-                pkmap[col_name]=True
+                pkmap[col_name] = True
 
         if len(pkmap) == 0:
-            raise PrimaryKeyNotFounded, tableName
+            raise PrimaryKeyNotFounded(tableName)
 
         for (field, nullAllowed, default, data_type, data_len,
              data_scale) in colData:
             # id is defined as primary key --> ok
-            # We let sqlobject raise error if the 'id' is used for another column
+            # We let sqlobject raise error if the 'id' is used
+            # for another column.
             field_name = field.lower()
             if (field_name == soClass.sqlmeta.idName) and pkmap[field_name]:
                 continue
 
-            colClass, kw = self.guessClass(data_type,data_len,data_scale)
+            colClass, kw = self.guessClass(data_type, data_len, data_scale)
             kw['name'] = field_name
             kw['dbName'] = field
 
-            if nullAllowed == 'Y' :
-                nullAllowed=False
+            if nullAllowed == 'Y':
+                nullAllowed = False
             else:
-                nullAllowed=True
+                nullAllowed = True
 
             kw['notNone'] = nullAllowed
             if default is not None:
                 kw['default'] = default
 
-            if keymap.has_key(field_name):
+            if field_name in keymap:
                 kw['foreignKey'] = keymap[field_name]
 
             results.append(colClass(**kw))
 
         return results
 
-    _numericTypes=['INTEGER', 'INT','SMALLINT']
-    _dateTypes=['DATE','TIME','TIMESTAMP']
+    _numericTypes = ['INTEGER', 'INT', 'SMALLINT']
+    _dateTypes = ['DATE', 'TIME', 'TIMESTAMP']
 
     def guessClass(self, t, flength, fscale=None):
         """
@@ -298,7 +313,7 @@ class MaxdbConnection(DBAPI):
         elif t in self._dateTypes:
             return col.DateTimeCol, {}
         elif t == 'FIXED':
-            return CurrencyCol,{'size':flength,
-                                'precision':fscale}
+            return col.CurrencyCol, {'size': flength,
+                                     'precision': fscale}
         else:
             return col.Col, {}
