@@ -1,6 +1,7 @@
-from sqlobject.dbconnection import DBAPI
-from sqlobject import col
 import re
+from sqlobject import col
+from sqlobject.dbconnection import DBAPI
+from sqlobject.compat import PY2
 
 
 class MSSQLConnection(DBAPI):
@@ -11,6 +12,8 @@ class MSSQLConnection(DBAPI):
 
     limit_re = re.compile('^\s*(select )(.*)', re.IGNORECASE)
 
+    odbc_keywords = ('Server', 'Port', 'User Id', 'Password', 'Database')
+
     def __init__(self, db, user, password='', host='localhost', port=None,
                  autoCommit=0, **kw):
         drivers = kw.pop('driver', None) or 'adodb,pymssql'
@@ -23,10 +26,23 @@ class MSSQLConnection(DBAPI):
                     import adodbapi as sqlmodule
                 elif driver == 'pymssql':
                     import pymssql as sqlmodule
+                elif driver == 'pyodbc':
+                    import pyodbc
+                    self.module = pyodbc
+                elif driver == 'pypyodbc':
+                    import pypyodbc
+                    self.module = pypyodbc
+                elif driver == 'odbc':
+                    try:
+                        import pyodbc
+                    except ImportError:
+                        import pypyodbc as pyodbc
+                    self.module = pyodbc
                 else:
                     raise ValueError(
                         'Unknown MSSQL driver "%s", '
-                        'expected adodb or pymssql' % driver)
+                        'expected adodb, pymssql, '
+                        'odbc, pyodbc or pypyodbc' % driver)
             except ImportError:
                 pass
             else:
@@ -34,9 +50,14 @@ class MSSQLConnection(DBAPI):
         else:
             raise ImportError(
                 'Cannot find an MSSQL driver, tried %s' % drivers)
-        self.module = sqlmodule
 
-        if sqlmodule.__name__ == 'adodbapi':
+        if driver in ('odbc', 'pyodbc', 'pypyodbc'):
+            self.make_odbc_conn_str(kw.pop('odbcdrv', 'SQL Server'),
+                                    db, host, port, user, password
+                                    )
+
+        elif driver in ('adodb', 'adodbapi'):
+            self.module = sqlmodule
             self.dbconnection = sqlmodule.connect
             # ADO uses unicode only (AFAIK)
             self.usingUnicodeStrings = True
@@ -60,28 +81,36 @@ class MSSQLConnection(DBAPI):
                 self.make_conn_str = lambda keys: conn_str % (
                     keys.host, keys.db, keys.user, keys.password)
 
-            kw.pop("sspi", None)
             kw.pop("ncli", None)
+            kw.pop("sspi", None)
 
-        else:  # pymssql
+        elif driver == 'pymssql':
+            self.module = sqlmodule
             self.dbconnection = sqlmodule.connect
             sqlmodule.Binary = lambda st: str(st)
             # don't know whether pymssql uses unicode
             self.usingUnicodeStrings = False
 
+            timeout = kw.pop('timeout', None)
+            if timeout:
+                timeout = int(timeout)
+            self.timeout = timeout
+
             def _make_conn_str(keys):
                 keys_dict = {}
                 for attr, value in (
+                    ('database', keys.db),
                     ('user', keys.user),
                     ('password', keys.password),
                     ('host', keys.host),
                     ('port', keys.port),
-                    ('database', keys.db),
+                    ('timeout', keys.timeout),
                 ):
                     if value:
                         keys_dict[attr] = value
                 return keys_dict
             self.make_conn_str = _make_conn_str
+        self.driver = driver
 
         self.autoCommit = int(autoCommit)
         self.user = user
@@ -111,17 +140,22 @@ class MSSQLConnection(DBAPI):
         return c.fetchone()[0]
 
     def makeConnection(self):
-        conn_descr = self.make_conn_str(self)
-        if isinstance(conn_descr, dict):
-            con = self.dbconnection(**conn_descr)
+        if self.driver in ('odbc', 'pyodbc', 'pypyodbc'):
+            self.debugWriter.write(
+                "ODBC connect string: " + self.odbc_conn_str)
+            conn = self.module.connect(self.odbc_conn_str)
         else:
-            con = self.dbconnection(conn_descr)
-        cur = con.cursor()
+            conn_descr = self.make_conn_str(self)
+            if isinstance(conn_descr, dict):
+                conn = self.dbconnection(**conn_descr)
+            else:
+                conn = self.dbconnection(conn_descr)
+        cur = conn.cursor()
         cur.execute('SET ANSI_NULLS ON')
         cur.execute("SELECT CAST('12345.21' AS DECIMAL(10, 2))")
         self.decimalSeparator = str(cur.fetchone()[0])[-3]
         cur.close()
-        return con
+        return conn
 
     HAS_IDENTITY = """
        select 1
@@ -163,7 +197,10 @@ class MSSQLConnection(DBAPI):
             else:
                 c.execute('SET IDENTITY_INSERT %s OFF' % table)
 
-        q = self._insertSQL(table, names, values)
+        if names and values:
+            q = self._insertSQL(table, names, values)
+        else:
+            q = "INSERT INTO %s DEFAULT VALUES" % table
         if self.debug:
             self.printDebug(conn, q, 'QueryIns')
         c.execute(q)
@@ -265,6 +302,9 @@ class MSSQLConnection(DBAPI):
                 if defaultText[0] == "'":
                     defaultText = defaultText[1:-1]
                 else:
+                    if t in ("int", "float", "numeric") and \
+                            defaultText[0] == "(":
+                        defaultText = defaultText[1:-1]
                     if t == "int":
                         defaultText = int(defaultText)
                     if t == "float":
@@ -289,9 +329,6 @@ class MSSQLConnection(DBAPI):
             option = "OFF"
         c = conn.cursor()
         c.execute("SET AUTOCOMMIT " + option)
-        # from mx.ODBC.Windows import SQL
-        # connection.setconnectoption(
-        # SQL.AUTOCOMMIT, SQL.AUTOCOMMIT_ON if auto else SQL.AUTOCOMMIT_OFF)
 
     # precision and scale is needed for decimal columns
     def guessClass(self, t, size, precision, scale):
@@ -327,12 +364,16 @@ class MSSQLConnection(DBAPI):
             9 - 2005
             10 - 2008
             11 - 2012
+            12 - 2014
+            13 - 2016
         """
         if self._server_version is not None:
             return self._server_version
         try:
             server_version = self.queryOne(
                 "SELECT SERVERPROPERTY('productversion')")[0]
+            if not PY2 and isinstance(server_version, bytes):
+                server_version = server_version.decode('ascii')
             server_version = server_version.split('.')[0]
             server_version = int(server_version)
         except:
