@@ -1,3 +1,5 @@
+import os
+
 from sqlobject import col, dberrors
 from sqlobject.compat import PY2
 from sqlobject.dbconnection import DBAPI
@@ -6,7 +8,10 @@ from sqlobject.dbconnection import DBAPI
 class ErrorMessage(str):
     def __new__(cls, e, append_msg=''):
         obj = str.__new__(cls, e.args[1] + append_msg)
-        obj.code = int(e.args[0])
+        try:
+            obj.code = int(e.args[0])
+        except ValueError:
+            obj.code = e.args[0]
         obj.module = e.__module__
         obj.exception = e.__class__.__name__
         return obj
@@ -20,11 +25,80 @@ class MySQLConnection(DBAPI):
     dbName = 'mysql'
     schemes = [dbName]
 
+    odbc_keywords = ('Server', 'Port', 'UID', 'Password', 'Database')
+
     def __init__(self, db, user, password='', host='localhost', port=0, **kw):
-        import MySQLdb, MySQLdb.constants.CR, MySQLdb.constants.ER  # noqa
-        self.module = MySQLdb
+        drivers = kw.pop('driver', None) or 'mysqldb'
+        for driver in drivers.split(','):
+            driver = driver.strip()
+            if not driver:
+                continue
+            try:
+                if driver.lower() in ('mysqldb', 'pymysql'):
+                    if driver.lower() == 'pymysql':
+                        import pymysql
+                        pymysql.install_as_MySQLdb()
+                    import MySQLdb
+                    if driver.lower() == 'mysqldb':
+                        if MySQLdb.version_info[:3] < (1, 2, 2):
+                            raise ValueError(
+                                'SQLObject requires MySQLdb 1.2.2 or later')
+                    import MySQLdb.constants.CR
+                    import MySQLdb.constants.ER
+                    self.module = MySQLdb
+                    if driver.lower() == 'mysqldb':
+                        self.CR_SERVER_GONE_ERROR = \
+                            MySQLdb.constants.CR.SERVER_GONE_ERROR
+                        self.CR_SERVER_LOST = \
+                            MySQLdb.constants.CR.SERVER_LOST
+                    else:
+                        self.CR_SERVER_GONE_ERROR = \
+                            MySQLdb.constants.CR.CR_SERVER_GONE_ERROR
+                        self.CR_SERVER_LOST = \
+                            MySQLdb.constants.CR.CR_SERVER_LOST
+                    self.ER_DUP_ENTRY = MySQLdb.constants.ER.DUP_ENTRY
+                elif driver == 'connector':
+                    import mysql.connector
+                    self.module = mysql.connector
+                    self.CR_SERVER_GONE_ERROR = \
+                        mysql.connector.errorcode.CR_SERVER_GONE_ERROR
+                    self.CR_SERVER_LOST = \
+                        mysql.connector.errorcode.CR_SERVER_LOST
+                    self.ER_DUP_ENTRY = mysql.connector.errorcode.ER_DUP_ENTRY
+                elif driver == 'oursql':
+                    import oursql
+                    self.module = oursql
+                    self.CR_SERVER_GONE_ERROR = \
+                        oursql.errnos['CR_SERVER_GONE_ERROR']
+                    self.CR_SERVER_LOST = oursql.errnos['CR_SERVER_LOST']
+                    self.ER_DUP_ENTRY = oursql.errnos['ER_DUP_ENTRY']
+                elif driver == 'pyodbc':
+                    import pyodbc
+                    self.module = pyodbc
+                elif driver == 'pypyodbc':
+                    import pypyodbc
+                    self.module = pypyodbc
+                elif driver == 'odbc':
+                    try:
+                        import pyodbc
+                    except ImportError:
+                        import pypyodbc as pyodbc
+                    self.module = pyodbc
+                else:
+                    raise ValueError(
+                        'Unknown MySQL driver "%s", '
+                        'expected mysqldb, connector, '
+                        'oursql, pymysql, '
+                        'odbc, pyodbc or pypyodbc' % driver)
+            except ImportError:
+                pass
+            else:
+                break
+        else:
+            raise ImportError(
+                'Cannot find a MySQL driver, tried %s' % drivers)
         self.host = host
-        self.port = port
+        self.port = port or 3306
         self.db = db
         self.user = user
         self.password = password
@@ -46,16 +120,23 @@ class MySQLConnection(DBAPI):
             self.dbEncoding = self.kw["charset"] = kw.pop("charset")
         else:
             self.dbEncoding = None
+        self.driver = driver
+
+        if driver in ('odbc', 'pyodbc', 'pypyodbc'):
+            self.make_odbc_conn_str(kw.pop('odbcdrv',
+                                           'MySQL ODBC 5.3 ANSI Driver'),
+                                    db, host, port, user, password
+                                    )
+            self.CR_SERVER_GONE_ERROR = 2006
+            self.CR_SERVER_LOST = 2013
+            self.ER_DUP_ENTRY = '23000'
 
         global mysql_Bin
         if not PY2 and mysql_Bin is None:
-            mysql_Bin = MySQLdb.Binary
-            MySQLdb.Binary = lambda x: mysql_Bin(x).decode(
+            mysql_Bin = self.module.Binary
+            self.module.Binary = lambda x: mysql_Bin(x).decode(
                 'ascii', errors='surrogateescape')
 
-        if self.module.version_info[:3] < (1, 2, 2):
-            raise ValueError('SQLObject requires MySQLdb 1.2.2 or later')
-
         self._server_version = None
         self._can_use_microseconds = None
         DBAPI.__init__(self, **kw)
@@ -64,37 +145,54 @@ class MySQLConnection(DBAPI):
     def _connectionFromParams(cls, user, password, host, port, path, args):
         return cls(db=path.strip('/'),
                    user=user or '', password=password or '',
-                   host=host or 'localhost', port=port or 0, **args)
+                   host=host or 'localhost', port=port, **args)
 
     def makeConnection(self):
         dbEncoding = self.dbEncoding
         if dbEncoding:
-            from MySQLdb.connections import Connection
-            if not hasattr(Connection, 'set_character_set'):
-                # monkeypatch pre MySQLdb 1.2.1
-                def character_set_name(self):
-                    return dbEncoding + '_' + dbEncoding
-                Connection.character_set_name = character_set_name
+            if self.driver.lower() in ('mysqldb', 'pymysql'):
+                from MySQLdb.connections import Connection
+                if not hasattr(Connection, 'set_character_set'):
+                    # monkeypatch pre MySQLdb 1.2.1
+                    def character_set_name(self):
+                        return dbEncoding + '_' + dbEncoding
+                    Connection.character_set_name = character_set_name
+        if self.driver == 'connector':
+            self.kw['consume_results'] = True
         try:
-            conn = self.module.connect(
-                host=self.host, port=self.port, db=self.db,
-                user=self.user, passwd=self.password, **self.kw)
-            if self.module.version_info[:3] >= (1, 2, 2):
-                # Attempt to reconnect. This setting is persistent.
-                conn.ping(True)
+            if self.driver in ('odbc', 'pyodbc', 'pypyodbc'):
+                self.debugWriter.write(
+                    "ODBC connect string: " + self.odbc_conn_str)
+                conn = self.module.connect(self.odbc_conn_str)
+            else:
+                conn = self.module.connect(
+                    host=self.host, port=self.port, db=self.db,
+                    user=self.user, passwd=self.password, **self.kw)
+                if self.driver != 'oursql':
+                    # Attempt to reconnect. This setting is persistent.
+                    conn.ping(True)
         except self.module.OperationalError as e:
             conninfo = ("; used connection string: "
                         "host=%(host)s, port=%(port)s, "
                         "db=%(db)s, user=%(user)s" % self.__dict__)
             raise dberrors.OperationalError(ErrorMessage(e, conninfo))
 
-        if hasattr(conn, 'autocommit'):
-            conn.autocommit(bool(self.autoCommit))
+        self._setAutoCommit(conn, bool(self.autoCommit))
 
         if dbEncoding:
-            if hasattr(conn, 'set_character_set'):  # MySQLdb 1.2.1 and later
+            if self.driver in ('odbc', 'pyodbc'):
+                conn.setdecoding(self.module.SQL_CHAR, encoding=dbEncoding)
+                conn.setdecoding(self.module.SQL_WCHAR, encoding=dbEncoding)
+                if PY2:
+                    conn.setencoding(str, encoding=dbEncoding)
+                    conn.setencoding(unicode, encoding=dbEncoding)  # noqa
+                else:
+                    conn.setencoding(encoding=dbEncoding)
+            elif hasattr(conn, 'set_character_set'):
                 conn.set_character_set(dbEncoding)
-            else:  # pre MySQLdb 1.2.1
+            elif self.driver == 'oursql':
+                conn.charset = dbEncoding
+            elif hasattr(conn, 'query'):
                 # works along with monkeypatching code above
                 conn.query("SET NAMES %s" % dbEncoding)
 
@@ -102,9 +200,19 @@ class MySQLConnection(DBAPI):
 
     def _setAutoCommit(self, conn, auto):
         if hasattr(conn, 'autocommit'):
-            conn.autocommit(auto)
+            try:
+                conn.autocommit(auto)
+            except TypeError:
+                # mysql-connector has autocommit as a property
+                conn.autocommit = auto
 
     def _executeRetry(self, conn, cursor, query):
+        if self.debug:
+            self.printDebug(conn, query, 'QueryR')
+        dbEncoding = self.dbEncoding
+        if dbEncoding and not isinstance(query, bytes) and (
+                self.driver == 'connector'):
+            query = query.encode(dbEncoding, 'surrogateescape')
         # When a server connection is lost and a query is attempted, most of
         # the time the query will raise a SERVER_LOST exception, then at the
         # second attempt to execute it, the mysql lib will reconnect and
@@ -121,8 +229,8 @@ class MySQLConnection(DBAPI):
             try:
                 return cursor.execute(query)
             except self.module.OperationalError as e:
-                if e.args[0] in (self.module.constants.CR.SERVER_GONE_ERROR,
-                                 self.module.constants.CR.SERVER_LOST):
+                if e.args[0] in (self.CR_SERVER_GONE_ERROR,
+                                 self.CR_SERVER_LOST):
                     if count == 2:
                         raise dberrors.OperationalError(ErrorMessage(e))
                     if self.debug:
@@ -131,14 +239,15 @@ class MySQLConnection(DBAPI):
                     raise dberrors.OperationalError(ErrorMessage(e))
             except self.module.IntegrityError as e:
                 msg = ErrorMessage(e)
-                if e.args[0] == self.module.constants.ER.DUP_ENTRY:
+                if e.args[0] == self.ER_DUP_ENTRY:
                     raise dberrors.DuplicateEntryError(msg)
                 else:
                     raise dberrors.IntegrityError(msg)
             except self.module.InternalError as e:
                 raise dberrors.InternalError(ErrorMessage(e))
             except self.module.ProgrammingError as e:
-                raise dberrors.ProgrammingError(ErrorMessage(e))
+                if e.args[0] is not None:
+                    raise dberrors.ProgrammingError(ErrorMessage(e))
             except self.module.DataError as e:
                 raise dberrors.DataError(ErrorMessage(e))
             except self.module.NotSupportedError as e:
@@ -167,7 +276,13 @@ class MySQLConnection(DBAPI):
             try:
                 id = c.lastrowid
             except AttributeError:
-                id = c.insert_id()
+                try:
+                    id = c.insert_id
+                except AttributeError:
+                    self._executeRetry(conn, c, "SELECT LAST_INSERT_ID();")
+                    id = c.fetchone()[0]
+                else:
+                    id = c.insert_id()
         if self.debugOutput:
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
@@ -205,7 +320,7 @@ class MySQLConnection(DBAPI):
             self.query('DESCRIBE %s' % (tableName))
             return True
         except dberrors.ProgrammingError as e:
-            if e.args[0].code == 1146:  # ER_NO_SUCH_TABLE
+            if e.args[0].code in (1146, '42S02'):  # ER_NO_SUCH_TABLE
                 return False
             raise
 
@@ -344,6 +459,9 @@ class MySQLConnection(DBAPI):
     def can_use_microseconds(self):
         if self._can_use_microseconds is not None:
             return self._can_use_microseconds
+        if os.environ.get('APPVEYOR') or os.environ.get('TRAVIS'):
+            self._can_use_microseconds = False
+            return False
         server_version = self.server_version()
         if server_version is None:
             return None
