@@ -1,9 +1,10 @@
-from sqlobject.dbconnection import DBAPI
 import re
 from sqlobject import col
 from sqlobject import dberrors
 from sqlobject import sqlbuilder
-from sqlobject.converters import registerConverter
+from sqlobject.compat import PY2
+from sqlobject.converters import registerConverter, sqlrepr
+from sqlobject.dbconnection import DBAPI
 
 
 class ErrorMessage(str):
@@ -13,7 +14,8 @@ class ErrorMessage(str):
             obj.code = getattr(e, 'pgcode', None)
             obj.error = getattr(e, 'pgerror', None)
         else:
-            obj.code = obj.error = None
+            obj.code = getattr(e, 'code', None)
+            obj.error = getattr(e, 'error', None)
         obj.module = e.__module__
         obj.exception = e.__class__.__name__
         return obj
@@ -25,6 +27,8 @@ class PostgresConnection(DBAPI):
     dbName = 'postgres'
     schemes = [dbName, 'postgresql']
 
+    odbc_keywords = ('Server', 'Port', 'UID', 'Password', 'Database')
+
     def __init__(self, dsn=None, host=None, port=None, db=None,
                  user=None, password=None, **kw):
         drivers = kw.pop('driver', None) or 'psycopg'
@@ -35,20 +39,40 @@ class PostgresConnection(DBAPI):
             try:
                 if driver == 'psycopg2':
                     import psycopg2 as psycopg
+                    self.module = psycopg
                 elif driver == 'psycopg1':
                     import psycopg
+                    self.module = psycopg
                 elif driver == 'psycopg':
                     try:
                         import psycopg2 as psycopg
                     except ImportError:
                         import psycopg
+                    self.module = psycopg
                 elif driver == 'pygresql':
                     import pgdb
                     self.module = pgdb
+                elif driver in ('py-postgresql', 'pypostgresql'):
+                    from postgresql.driver import dbapi20
+                    self.module = dbapi20
+                elif driver == 'pyodbc':
+                    import pyodbc
+                    self.module = pyodbc
+                elif driver == 'pypyodbc':
+                    import pypyodbc
+                    self.module = pypyodbc
+                elif driver == 'odbc':
+                    try:
+                        import pyodbc
+                    except ImportError:
+                        import pypyodbc as pyodbc
+                    self.module = pyodbc
                 else:
                     raise ValueError(
                         'Unknown PostgreSQL driver "%s", '
-                        'expected psycopg2, psycopg1 or pygresql' % driver)
+                        'expected psycopg, psycopg2, psycopg1, '
+                        'pygresql, pypostgresql, '
+                        'odbc, pyodbc or pypyodbc' % driver)
             except ImportError:
                 pass
             else:
@@ -56,69 +80,89 @@ class PostgresConnection(DBAPI):
         else:
             raise ImportError(
                 'Cannot find a PostgreSQL driver, tried %s' % drivers)
-        if driver.startswith('psycopg'):
-            self.module = psycopg
-            # Register a converter for psycopg Binary type.
-            registerConverter(type(psycopg.Binary('')),
-                              PsycoBinaryConverter)
 
+        if driver.startswith('psycopg'):
+            # Register a converter for psycopg Binary type.
+            registerConverter(type(self.module.Binary('')),
+                              PsycoBinaryConverter)
+        elif driver in ('pygresql', 'py-postgresql', 'pypostgresql'):
+            registerConverter(type(self.module.Binary(b'')),
+                              PostgresBinaryConverter)
+        elif driver in ('odbc', 'pyodbc', 'pypyodbc'):
+            registerConverter(bytearray, OdbcBinaryConverter)
+
+        self.db = db
         self.user = user
+        self.password = password
         self.host = host
         self.port = port
-        self.db = db
-        self.password = password
-        self.dsn_dict = dsn_dict = {}
-        if host:
-            dsn_dict["host"] = host
-        if port:
-            if driver == 'pygresql':
-                dsn_dict["host"] = "%s:%d" % (host, port)
-            else:
-                if psycopg.__version__.split('.')[0] == '1':
+        if driver in ('odbc', 'pyodbc', 'pypyodbc'):
+            self.make_odbc_conn_str(kw.pop('odbcdrv', 'PostgreSQL ANSI'),
+                                    db, host, port, user, password
+                                    )
+            sslmode = kw.pop("sslmode", None)
+            if sslmode:
+                self.odbc_conn_str += ';sslmode=require'
+        else:
+            self.dsn_dict = dsn_dict = {}
+            if host:
+                dsn_dict["host"] = host
+            if port:
+                if driver == 'pygresql':
+                    dsn_dict["host"] = "%s:%d" % (host, port)
+                elif driver.startswith('psycopg') and \
+                        psycopg.__version__.split('.')[0] == '1':
                     dsn_dict["port"] = str(port)
                 else:
                     dsn_dict["port"] = port
-        if db:
-            dsn_dict["database"] = db
-        if user:
-            dsn_dict["user"] = user
-        if password:
-            dsn_dict["password"] = password
-        sslmode = kw.pop("sslmode", None)
-        if sslmode:
-            dsn_dict["sslmode"] = sslmode
-        self.use_dsn = dsn is not None
-        if dsn is None:
-            if driver == 'pygresql':
-                dsn = ''
-                if host:
-                    dsn += host
-                dsn += ':'
-                if db:
-                    dsn += db
-                dsn += ':'
-                if user:
-                    dsn += user
-                dsn += ':'
-                if password:
-                    dsn += password
-            else:
-                dsn = []
-                if db:
-                    dsn.append('dbname=%s' % db)
-                if user:
-                    dsn.append('user=%s' % user)
-                if password:
-                    dsn.append('password=%s' % password)
-                if host:
-                    dsn.append('host=%s' % host)
-                if port:
-                    dsn.append('port=%d' % port)
-                if sslmode:
-                    dsn.append('sslmode=%s' % sslmode)
-                dsn = ' '.join(dsn)
+            if db:
+                dsn_dict["database"] = db
+            if user:
+                dsn_dict["user"] = user
+            if password:
+                dsn_dict["password"] = password
+            sslmode = kw.pop("sslmode", None)
+            if sslmode:
+                dsn_dict["sslmode"] = sslmode
+            self.use_dsn = dsn is not None
+            if dsn is None:
+                if driver == 'pygresql':
+                    dsn = ''
+                    if host:
+                        dsn += host
+                    dsn += ':'
+                    if db:
+                        dsn += db
+                    dsn += ':'
+                    if user:
+                        dsn += user
+                    dsn += ':'
+                    if password:
+                        dsn += password
+                else:
+                    dsn = []
+                    if db:
+                        dsn.append('dbname=%s' % db)
+                    if user:
+                        dsn.append('user=%s' % user)
+                    if password:
+                        dsn.append('password=%s' % password)
+                    if host:
+                        dsn.append('host=%s' % host)
+                    if port:
+                        dsn.append('port=%d' % port)
+                    if sslmode:
+                        dsn.append('sslmode=%s' % sslmode)
+                    dsn = ' '.join(dsn)
+            if driver in ('py-postgresql', 'pypostgresql'):
+                if host and host.startswith('/'):
+                    dsn_dict["host"] = dsn_dict["port"] = None
+                    dsn_dict["unix"] = host
+                else:
+                    if "unix" in dsn_dict:
+                        del dsn_dict["unix"]
+            self.dsn = dsn
         self.driver = driver
-        self.dsn = dsn
         self.unicodeCols = kw.pop('unicodeCols', False)
         self.schema = kw.pop('schema', None)
         self.dbEncoding = kw.pop("charset", None)
@@ -144,7 +188,11 @@ class PostgresConnection(DBAPI):
 
     def makeConnection(self):
         try:
-            if self.use_dsn:
+            if self.driver in ('odbc', 'pyodbc', 'pypyodbc'):
+                self.debugWriter.write(
+                    "ODBC connect string: " + self.odbc_conn_str)
+                conn = self.module.connect(self.odbc_conn_str)
+            elif self.use_dsn:
                 conn = self.module.connect(self.dsn)
             else:
                 conn = self.module.connect(**self.dsn_dict)
@@ -162,6 +210,14 @@ class PostgresConnection(DBAPI):
             self._executeRetry(conn, c, "SET search_path TO " + self.schema)
         dbEncoding = self.dbEncoding
         if dbEncoding:
+            if self.driver in ('odbc', 'pyodbc'):
+                conn.setdecoding(self.module.SQL_CHAR, encoding=dbEncoding)
+                conn.setdecoding(self.module.SQL_WCHAR, encoding=dbEncoding)
+                if PY2:
+                    conn.setencoding(str, encoding=dbEncoding)
+                    conn.setencoding(unicode, encoding=dbEncoding)  # noqa
+                else:
+                    conn.setencoding(encoding=dbEncoding)
             self._executeRetry(conn, c,
                                "SET client_encoding TO '%s'" % dbEncoding)
         return conn
@@ -175,20 +231,31 @@ class PostgresConnection(DBAPI):
             raise dberrors.OperationalError(ErrorMessage(e))
         except self.module.IntegrityError as e:
             msg = ErrorMessage(e)
-            if e.pgcode == '23505':
+            if getattr(e, 'code', -1) == '23505' or \
+                    getattr(e, 'pgcode', -1) == '23505' or \
+                    getattr(e, 'sqlstate', -1) == '23505' or \
+                    e.args[0] == '23505':
                 raise dberrors.DuplicateEntryError(msg)
             else:
                 raise dberrors.IntegrityError(msg)
         except self.module.InternalError as e:
             raise dberrors.InternalError(ErrorMessage(e))
         except self.module.ProgrammingError as e:
-            raise dberrors.ProgrammingError(ErrorMessage(e))
+            msg = ErrorMessage(e)
+            if (len(e.args) >= 2) and e.args[1] == '23505':
+                raise dberrors.DuplicateEntryError(msg)
+            else:
+                raise dberrors.ProgrammingError(msg)
         except self.module.DataError as e:
             raise dberrors.DataError(ErrorMessage(e))
         except self.module.NotSupportedError as e:
             raise dberrors.NotSupportedError(ErrorMessage(e))
         except self.module.DatabaseError as e:
-            raise dberrors.DatabaseError(ErrorMessage(e))
+            msg = ErrorMessage(e)
+            if 'duplicate key value violates unique constraint' in msg:
+                raise dberrors.DuplicateEntryError(msg)
+            else:
+                raise dberrors.DatabaseError(msg)
         except self.module.InterfaceError as e:
             raise dberrors.InterfaceError(ErrorMessage(e))
         except self.module.Warning as e:
@@ -200,6 +267,11 @@ class PostgresConnection(DBAPI):
         table = soInstance.sqlmeta.table
         idName = soInstance.sqlmeta.idName
         c = conn.cursor()
+        if id is None and self.driver in ('py-postgresql', 'pypostgresql'):
+            sequenceName = soInstance.sqlmeta.idSequence or \
+                '%s_%s_seq' % (table, idName)
+            self._executeRetry(conn, c, "SELECT NEXTVAL('%s')" % sequenceName)
+            id = c.fetchone()[0]
         if id is not None:
             names = [idName] + names
             values = [id] + values
@@ -458,7 +530,34 @@ class PostgresConnection(DBAPI):
         self._createOrDropDatabase(op="DROP")
 
 
-# Converter for psycopg Binary type.
+# Converter for Binary types
 def PsycoBinaryConverter(value, db):
     assert db == 'postgres'
     return str(value)
+
+
+if PY2:
+    def escape_bytea(value):
+        return ''.join(
+            ['\\' + (x[1:].rjust(3, '0'))
+                for x in (oct(ord(c)) for c in value)]
+        )
+else:
+    def escape_bytea(value):
+        return ''.join(
+            ['\\' + (x[2:].rjust(3, '0'))
+                for x in (oct(ord(c)) for c in value.decode('latin1'))]
+        )
+
+
+def PostgresBinaryConverter(value, db):
+    assert db == 'postgres'
+    return sqlrepr(escape_bytea(value), db)
+
+
+def OdbcBinaryConverter(value, db):
+    assert db == 'postgres'
+    value = bytes(value)
+    if not PY2:
+        value = value.decode('latin1')
+    return value
