@@ -1,18 +1,26 @@
-from sqlobject import *
-from sqlobject.tests.dbtest import *
+import pytest
+from sqlobject import BLOBCol, SQLObject
+from sqlobject.compat import PY2
+from sqlobject.tests.dbtest import setupClass, supports
+
 
 ########################################
-## BLOB columns
+# BLOB columns
 ########################################
 
+
 class ImageData(SQLObject):
-    image = BLOBCol(default='emptydata', length=65535)
+    image = BLOBCol(default=b'emptydata', length=256)
+
 
 def test_BLOBCol():
     if not supports('blobData'):
-        return
+        pytest.skip("blobData isn't supported")
     setupClass(ImageData)
-    data = ''.join([chr(x) for x in range(256)])
+    if PY2:
+        data = ''.join([chr(x) for x in range(256)])
+    else:
+        data = bytes(range(256))
 
     prof = ImageData()
     prof.image = data
