@@ -1,15 +1,15 @@
-from sqlobject import declarative
+import pytest
+
 from sqlobject import boundattributes
-import py.test
+from sqlobject import declarative
 
-pytestmark = py.test.mark.skipif('True')
+pytestmark = pytest.mark.skipif('True')
 
-class TestMe(object):
 
-    #__metaclass__ = declarative.DeclarativeMeta
-    #__classinit__ = boundattributes.bind_attributes_local
+class SOTestMe(object):
     pass
 
+
 class AttrReplace(boundattributes.BoundAttribute):
 
     __unpackargs__ = ('replace',)
@@ -28,22 +28,28 @@ class AttrReplace(boundattributes.BoundAttribute):
         self.replace.attrs = attrs
         return self.replace
 
+
 class Holder:
     def __init__(self, name):
         self.holder_name = name
+
     def __repr__(self):
         return '<Holder %s>' % self.holder_name
 
+
 def test_1():
     v1 = Holder('v1')
     v2 = Holder('v2')
     v3 = Holder('v3')
+
     class V2Class(AttrReplace):
         arg1 = 'nothing'
         arg2 = ['something']
-    class A1(TestMe):
+
+    class A1(SOTestMe):
         a = AttrReplace(v1)
         v = V2Class(v2)
+
         class inline(AttrReplace):
             replace = v3
             arg3 = 'again'
