@@ -1,21 +1,31 @@
-from sqlobject.constraints import *
-from sqlobject.tests.dbtest import *
+from sqlobject.compat import PY2
+from sqlobject.constraints import BadValue, InList, MaxLength, \
+    isFloat, isInt, isString, notNull
+from sqlobject.tests.dbtest import Dummy, raises
+
+if not PY2:
+    # alias for python 3 compatability
+    long = int
+
 
 def test_constraints():
     obj = 'Test object'
     col = Dummy(name='col')
     isString(obj, col, 'blah')
     raises(BadValue, isString, obj, col, 1)
-    # @@: Should this really be an error?
-    raises(BadValue, isString, obj, col, u'test!')
-    #isString(obj, col, u'test!')
+    if PY2:
+        # @@: Should this really be an error?
+        raises(BadValue, isString, obj, col, u'test!')
+    else:
+        raises(BadValue, isString, obj, col, b'test!')
+    # isString(obj, col, u'test!')
 
     raises(BadValue, notNull, obj, col, None)
     raises(BadValue, isInt, obj, col, 1.1)
     isInt(obj, col, 1)
-    isInt(obj, col, 1L)
+    isInt(obj, col, long(1))
     isFloat(obj, col, 1)
-    isFloat(obj, col, 1L)
+    isFloat(obj, col, long(1))
     isFloat(obj, col, 1.2)
     raises(BadValue, isFloat, obj, col, '1.0')
 
