@@ -1,5 +1,6 @@
 from datetime import timedelta
 
+from sqlobject.col import SODateTimeCol, SOTimeCol
 from sqlobject.converters import registerConverter, sqlrepr, \
     quote_str, unquote_str
 from sqlobject.sqlbuilder import SQLExpression, SQLObjectField, \
@@ -8,30 +9,30 @@ from sqlobject.sqlbuilder import SQLExpression, SQLObjectField, \
     _LikeQuoted
 
 
-class TestClass:
+class SOTestClass:
 
     def __repr__(self):
-        return '<TestClass>'
+        return '<SOTestClass>'
 
 
-def TestClassConverter(value, db):
+def SOTestClassConverter(value, db):
     return repr(value)
 
-registerConverter(TestClass, TestClassConverter)
+registerConverter(SOTestClass, SOTestClassConverter)
 
 
-class NewTestClass:
+class NewSOTestClass:
 
     __metaclass__ = type
 
     def __repr__(self):
-        return '<NewTestClass>'
+        return '<NewSOTestClass>'
 
 
-def NewTestClassConverter(value, db):
+def NewSOTestClassConverter(value, db):
     return repr(value)
 
-registerConverter(NewTestClass, NewTestClassConverter)
+registerConverter(NewSOTestClass, NewSOTestClassConverter)
 
 
 def _sqlrepr(self, db):
@@ -110,23 +111,34 @@ def test_bool():
 
 def test_datetime():
     from datetime import datetime, date, time
-    assert sqlrepr(datetime(2005, 7, 14, 13, 31, 2)) == \
-        "'2005-07-14 13:31:02.000000'"
+    if SODateTimeCol.datetimeFormat.find('.%f') > 0:
+        assert sqlrepr(datetime(2005, 7, 14, 13, 31, 2)) == \
+            "'2005-07-14 13:31:02.000000'"
+    else:
+        assert sqlrepr(datetime(2005, 7, 14, 13, 31, 2)) == \
+            "'2005-07-14 13:31:02'"
     assert sqlrepr(date(2005, 7, 14)) == "'2005-07-14'"
-    assert sqlrepr(time(13, 31, 2)) == "'13:31:02.000000'"
+    if SOTimeCol.timeFormat.find('.%f') > 0:
+        assert sqlrepr(time(13, 31, 2)) == "'13:31:02.000000'"
+    else:
+        assert sqlrepr(time(13, 31, 2)) == "'13:31:02'"
     # now dates before 1900
-    assert sqlrepr(datetime(1428, 7, 14, 13, 31, 2)) == \
-        "'1428-07-14 13:31:02.000000'"
+    if SODateTimeCol.datetimeFormat.find('.%f') > 0:
+        assert sqlrepr(datetime(1428, 7, 14, 13, 31, 2)) == \
+            "'1428-07-14 13:31:02.000000'"
+    else:
+        assert sqlrepr(datetime(1428, 7, 14, 13, 31, 2)) == \
+            "'1428-07-14 13:31:02'"
     assert sqlrepr(date(1428, 7, 14)) == "'1428-07-14'"
 
 
 def test_instance():
-    instance = TestClass()
+    instance = SOTestClass()
     assert sqlrepr(instance) == repr(instance)
 
 
 def test_newstyle():
-    instance = NewTestClass()
+    instance = NewSOTestClass()
     assert sqlrepr(instance) == repr(instance)
 
 
