@@ -1,22 +1,26 @@
-import py.test
+import pytest
 from sqlobject import SQLObject
 from sqlobject.tests.dbtest import getConnection, setupClass
 
 
-class TestSOListMySQL(SQLObject):
+try:
+    connection = getConnection()
+except (AttributeError, NameError):
+    # The module was imported during documentation building
+    pass
+else:
+    if connection.dbName != "mysql":
+        pytestmark = pytest.mark.skip('')
+
+
+class SOTestSOListMySQL(SQLObject):
     pass
 
 
 def test_list_databases():
-    connection = getConnection()
-    if connection.dbName != "mysql":
-        py.test.skip("These tests require MySQL")
     assert connection.db in connection.listDatabases()
 
 
 def test_list_tables():
-    connection = getConnection()
-    if connection.dbName != "mysql":
-        py.test.skip("These tests require MySQL")
-    setupClass(TestSOListMySQL)
-    assert TestSOListMySQL.sqlmeta.table in connection.listTables()
+    setupClass(SOTestSOListMySQL)
+    assert SOTestSOListMySQL.sqlmeta.table in connection.listTables()
